// Win32 DF by Tom Grandgent (tom@grandgent.com)
//
// v1.0 - 9/6/1999  - Original release
// v1.1 - 3/27/2000 - Corrected minor display problems
// v1.2 - 5/29/2000 - Added local fixed disk totals
// v1.3 - 7/25/2002 - Improved display of cluster sizes over 64KB
//                    Added display of network share paths

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <winnetwk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Network info
HANDLE enumHandle;
NETRESOURCE nr[256];
DWORD netcount = 0;
int enumedNetwork = 0;

// Attempts to enumerate connected network shares
int enum_network(void)
{
	DWORD bufsize = sizeof(NETRESOURCE) * 256;
	unsigned int i;

	// Enumerate connected disks
	if (WNetOpenEnum(RESOURCE_CONNECTED, RESOURCETYPE_DISK, RESOURCEUSAGE_CONNECTABLE, NULL, &enumHandle) == NO_ERROR)
		enumedNetwork = 1;
	else
		return 1;

	netcount = -1;
	i = WNetEnumResource(enumHandle, &netcount, &nr, &bufsize);
	if (i != NO_ERROR)
	{
		netcount = 0;
		return 1;
	}

	return 0;
}

int main(int argc, char *argv[])
{
	unsigned int i;
	char drive;
	char dir[64];
	char tmpbuf[2048];
	int useKB = 0;

	// Info gathered from GetDiskFreeSpaceEx() and GetDiskFreeSpace()
	ULARGE_INTEGER utotal_bytes, ufree_bytes, uquota_bytes;
	DWORD sectors_per_cluster, bytes_per_sector, free_clusters, total_clusters, cluster_size, total_cluster_size = 0;
	char clusterSizeString[64];
	
	// Per-drive stats
	__int64 total_bytes, free_bytes, used_bytes;
	float pct_free;
	// Total stats
	__int64 ftotal_bytes = 0, ffree_bytes = 0, fused_bytes = 0;
	float fpct_free;
	// Total local fixed disk stats
	__int64 fxtotal_bytes = 0, fxfree_bytes = 0, fxused_bytes = 0;
	float fxpct_free;

	// Drive info
	DWORD drive_type_dword;
	char drive_type[128];
	char volume_name[256];
	char file_system[128];
	int num_drives = 0;

	printf("\nWin32 DF 1.4 - (c) Tom Grandgent - 9/6/99 - based on DF 1.7 by Kai Uwe Rommel\n\n");

	// Check parameters
	for (i = 1; i < (unsigned int) argc; i++)
	{
		if (!stricmp(argv[i], "/?") || !stricmp(argv[i], "-?"))
		{
			printf("Usage: %s [-k]\n\n", argv[0]);
			printf("Options:\n");
			printf("    -k Display values in kilobytes\n");
			return 0;
		}
		if (!stricmp(argv[i], "/k") || !stricmp(argv[i], "-k"))
		{
			useKB = 1;
		}
	}

	// Enumerate network shares
	enum_network();

	// Print headings
	if (useKB)
	{
		printf("   %% free     free         used        total     cluster  fs    type    label  \n");
		printf( "-- ------ ------------ ------------ ------------ ------- ----- ------ ---------\n");
	}
	else
	{
		printf("   %% free    free       used      total    cluster  fs    type    label  \n");
		printf( "-- ------ ---------- ---------- ---------- ------- ----- ------ ---------\n");
	}
	
	// Loop through all of the drive letters from C to Z
	for (drive = 'C'; drive <= 'Z'; drive++)
	{
		// Secret /d argument is just extra verbose
		if (argc > 1 && !strcmp(argv[1], "/d"))
			printf("Checking %c:\\...\n", drive);
		// Form a directory name from the drive letter and ask Windows for disk space info
		sprintf(dir, "%c:\\", drive);
		if (GetDiskFreeSpaceEx(dir, &uquota_bytes, &utotal_bytes, &ufree_bytes))
		{
			num_drives++;
			// Do calculations for the disk size
			total_bytes = utotal_bytes.QuadPart;
			free_bytes = ufree_bytes.QuadPart;
			total_bytes /= 1000;
			free_bytes /= 1000;
			used_bytes = total_bytes - free_bytes;
			ftotal_bytes += total_bytes;
			ffree_bytes += free_bytes;
			fused_bytes += used_bytes;
			if (total_bytes)
				pct_free = (float) free_bytes / (float) total_bytes;
			else
				pct_free = 0.0f;
			pct_free *= 100.0f;
			// Get the drive type
			drive_type_dword = GetDriveType(dir);
			switch (drive_type_dword)
			{
				case DRIVE_UNKNOWN: strcpy(drive_type, "???"); break;
				case DRIVE_REMOVABLE: strcpy(drive_type, "Remov."); break;
				case DRIVE_FIXED: 
					strcpy(drive_type, "Fixed"); 
					fxtotal_bytes += total_bytes;
					fxfree_bytes += free_bytes;
					fxused_bytes += used_bytes;
					break;
				case DRIVE_REMOTE: strcpy(drive_type, "Remote"); break;
				case DRIVE_CDROM: strcpy(drive_type, "CD-ROM"); break;
				case DRIVE_RAMDISK: strcpy(drive_type, "RAMDsk"); break;
				default: strcpy(drive_type, "Unknown");
			}
			// Get the drive geometry
			GetDiskFreeSpace(dir, &sectors_per_cluster, &bytes_per_sector, &free_clusters, &total_clusters);
			cluster_size = bytes_per_sector * sectors_per_cluster;
			if (cluster_size <= 65536)
				itoa(cluster_size, clusterSizeString, 10);
			else
			if (cluster_size < 1024 * 1024)
				sprintf(clusterSizeString, "%d k", cluster_size / 1024);
			else
				sprintf(clusterSizeString, "%d MB", cluster_size / (1024 * 1024));
			total_cluster_size += cluster_size;
			// Get the drive volume name
			GetVolumeInformation(dir, volume_name, sizeof(volume_name), NULL, NULL, NULL, file_system, sizeof(file_system));
			//volume_name[9] = '\0';		// Terminate volume label if it's too long
			if (!stricmp(file_system, "VBoxSharedFolderFS")) // Abbreviate this long FS name
				file_system[4] = '\0';
			if (enumedNetwork)
			{
				//printf("netcount: %s\n", nr[0].lpLocalName);
				for (i = 0; i < netcount; i++)
					if (nr[i].lpLocalName && nr[i].lpLocalName[1] == ':' && 
						toupper(nr[i].lpLocalName[0]) == drive)
					{
						strcpy(volume_name, nr[i].lpRemoteName);
						break;
					}
			}
			/*for (i = 0; i < 110; i++)
				tmpbuf[i] = 'A' + rand() % 26;
			tmpbuf[i] = '\0';
			strcpy(volume_name, tmpbuf);*/
			// Display the results
			if (useKB)
			{
				printf("%c: %3.0f %%  %9I64d k  %9I64d k  %9I64d k  %6s  %-5s %-6s %-9s\n", 
					drive, pct_free, free_bytes, used_bytes, total_bytes, clusterSizeString,
					file_system, drive_type, volume_name);
			}
			else
			{
				printf("%c: %3.0f %%  %7I64d M  %7I64d M  %7I64d M  %6s  %-5s %-6s %-9s\n", 
					drive, pct_free, free_bytes / 1000, used_bytes / 1000, total_bytes / 1000, clusterSizeString,
					file_system, drive_type, volume_name);
			}
		}
	}
	if (useKB)
	{
		printf( "-- ------ ------------ ------------ ------------\n");
	}
	else
	{
		printf( "-- ------ ---------- ---------- ----------\n");
	}
	// Display total results
	if (ftotal_bytes)
		fpct_free = (float) ffree_bytes / (float) ftotal_bytes;
	else
		fpct_free = 0;
	fpct_free *= 100.0f;
	if (fxtotal_bytes)
		fxpct_free = (float) fxfree_bytes / (float) fxtotal_bytes;
	else
		fxpct_free = 0;
	fxpct_free *= 100.0f;
	if (fxtotal_bytes != ftotal_bytes)
	{
		if (useKB)
		{
			printf("Fix %2.0f %%  %9I64d k  %9I64d k  %9I64d k\n", 
				fxpct_free, fxfree_bytes, fxused_bytes, fxtotal_bytes);
		}
		else
		{
			printf("Fix %2.0f %%  %7I64d M  %7I64d M  %7I64d M\n", 
				fxpct_free, fxfree_bytes / 1000, fxused_bytes / 1000, fxtotal_bytes / 1000);
		}
	}
	if (useKB)
	{
		printf("All %2.0f %%  %9I64d k  %9I64d k  %9I64d k\n", 
			fpct_free, ffree_bytes, fused_bytes, ftotal_bytes);
	}
	else
	{
		printf("All %2.0f %%  %7I64d M  %7I64d M  %7I64d M\n", 
			fpct_free, ffree_bytes / 1000, fused_bytes / 1000, ftotal_bytes / 1000);
	}

	// Clean up network enumeration data
	if (enumedNetwork)
		WNetCloseEnum(enumHandle);

	return 0;
}